#define SCLK                5
#define SDIO                6
#define NCS                 4

#define PRODUCT_ID          0x00 // should be 0x12
#define PRODUCTID2          0x3e
#define REVISION_ID         0x01
#define MOTION_REG					0x02
#define DELTA_Y_REG         0x03
#define DELTA_X_REG         0x04
#define SQUAL_REG           0x05
#define MAXIMUM_PIXEL_REG   0x08
#define MINIMUM_PIXEL_REG   0x0a
#define PIXEL_SUM_REG       0x09
#define PIXEL_DATA_REG      0x0b
#define SHUTTER_UPPER_REG   0x06
#define SHUTTER_LOWER_REG   0x07
#define RESET		    0x3a
#define CPI500v		    0x00
#define CPI1000v	    0x01

#define NUM_PIXELS          361

byte pix[361];

void setup()
{
  Serial.begin(115200);

  pinMode(SDIO, OUTPUT);
  pinMode(SCLK, OUTPUT);
  pinMode(NCS, OUTPUT);
  
  sync();

  ADNS_write(RESET, 0x5a);
  delay(100); // From NRESET pull high to valid motion, assuming VDD and motion is present.


  byte productId = ADNS_read(PRODUCT_ID);
  byte revisionId = ADNS_read(REVISION_ID);
  Serial.print("Found productId ");
  Serial.print(productId, HEX);
  Serial.print(", rev. ");
  Serial.println(revisionId, HEX);
}

void loop()
{
  ADNS_read(MOTION_REG); // Freezes DX and DY until they are read or MOTION is read again.
  char dx = ADNS_read(DELTA_X_REG);
  char dy = ADNS_read(DELTA_Y_REG);
  Serial.print("DELTA:");
  Serial.print(dx, DEC);
  Serial.print(" ");
  Serial.println(dy, DEC);


  pixelGrab();

  Serial.print("FRAME:");
  for(int i = 0; i < NUM_PIXELS; i++) {
    byte p = pix[i];
    if( p < 0x10 )
      Serial.print("0");
    Serial.print(p, HEX);
  }
  Serial.println();
}

void pixelGrab()
{
 	//ADNS_write(PIXEL_DATA_REG, 0); // start reading
 	//delayMicroseconds(20);
  
  int grabCount = 0; 
  while( grabCount < NUM_PIXELS )
  {
    byte data = ADNS_read(PIXEL_DATA_REG);
    if( (data & 0x80) != 0 ) { // Data is valid
      pix[grabCount++] = data&0x7f;
    }
  }
}

void sync() {
  digitalWrite(NCS, LOW);
  delayMicroseconds(2);
  digitalWrite(NCS, HIGH);
}

void ADNS_write_old(unsigned char addr, unsigned char data) {
  char temp;
  int n;

  digitalWrite(NCS, LOW);//nADNSCS = 0; // select the chip

  temp = addr;
  digitalWrite(SCLK, LOW);//SCK = 0;					 // start clock low
  pinMode(SDIO, OUTPUT);//DATA_OUT; // set data line for output
  for (n=0; n<8; n++) {
    digitalWrite(SCLK, LOW);//SCK = 0;
    pinMode(SDIO, OUTPUT);
    delayMicroseconds(1);
    if (temp & 0x80)
      digitalWrite(SDIO, HIGH);//SDOUT = 1;
    else
      digitalWrite(SDIO, LOW);//SDOUT = 0;
    temp = (temp << 1);
    digitalWrite(SCLK, HIGH);//SCK = 1;
    delayMicroseconds(1);//delayMicroseconds(1);			 // short clock pulse
  }
  temp = data;
  for (n=0; n<8; n++) {
    digitalWrite(SCLK, LOW);//SCK = 0;
    delayMicroseconds(1);
    if (temp & 0x80)
      digitalWrite(SDIO, HIGH);//SDOUT = 1;
    else
      digitalWrite(SDIO, LOW);//SDOUT = 0;
    temp = (temp << 1);
    digitalWrite(SCLK, HIGH);//SCK = 1;
    delayMicroseconds(1);			 // short clock pulse
  }
  delayMicroseconds(20);
  digitalWrite(NCS, HIGH);//nADNSCS = 1; // de-select the chip
}

byte ADNS_read_old(unsigned char addr) {
  byte temp;
  int n;

  digitalWrite(NCS, LOW);//nADNSCS = 0;				 // select the chip
  temp = addr;
  digitalWrite(SCLK, OUTPUT); //SCK = 0;					 // start clock low
  pinMode(SDIO, OUTPUT); //DATA_OUT;					 // set data line for output
  for (n=0; n<8; n++) {

    digitalWrite(SCLK, LOW);//SCK = 0;
    pinMode(SDIO, OUTPUT); //DATA_OUT;
    if (temp & 0x80) {
      digitalWrite(SDIO, HIGH);//SDOUT = 1;
    } 
    else {
      digitalWrite(SDIO, LOW);//SDOUT = 0;
    }
    temp = (temp << 1);
    delayMicroseconds(1);
    digitalWrite(SCLK, HIGH); //SCK = 1;
    delayMicroseconds(1);			 // short clock pulse
  }
  
  temp = 0; // This is a read, switch to input
  pinMode(SDIO, INPUT); //DATA_IN;
  for (n=0; n<8; n++) {		 // read back the data
    digitalWrite(SCLK, LOW);
    if(digitalRead(SDIO)) {// got a '1'
      temp |= 0x1;
    }
    if( n != 7) temp = (temp << 1); // shift left
    digitalWrite(SCLK, HIGH);
  }
  
  digitalWrite(NCS, HIGH);// de-select the chip
  return temp;
}


byte ADNS_read(byte address) {
  digitalWrite(NCS, LOW);// select the chip
  pinMode (SDIO, OUTPUT);

  for (byte i=128; i >0 ; i >>= 1) {
    digitalWrite (SCLK, LOW);
    digitalWrite (SDIO, (address & i) != 0 ? HIGH : LOW);
    digitalWrite (SCLK, HIGH);
  }

  pinMode (SDIO, INPUT);

  //delayMicroseconds(20); // tSWR = 20us min.

  byte res = 0;
  for (byte i=128; i >0 ; i >>= 1) {
    digitalWrite (SCLK, LOW);
    digitalWrite (SCLK, HIGH);
    if( digitalRead (SDIO) == HIGH )
      res |= i;
  }

  digitalWrite(NCS, HIGH);// de-select the chip
  return res;
}

void ADNS_write(byte address, byte data) {
  digitalWrite(NCS, LOW);// select the chip
  pinMode (SDIO, OUTPUT);

  for (byte i = 128; i > 0 ; i >>= 1) {
    digitalWrite (SCLK, LOW);
    digitalWrite (SDIO, (address & i) != 0 ? HIGH : LOW);
    digitalWrite (SCLK, HIGH);
  }
  
  //delayMicroseconds(30); // tSWW = 30us min.

  for (byte i = 128; i > 0 ; i >>= 1) {
    digitalWrite (SCLK, LOW);
    digitalWrite (SDIO, (data & i) != 0 ? HIGH : LOW);
    digitalWrite (SCLK, HIGH);
  }

  digitalWrite(NCS, HIGH);// de-select the chip
}
